@@ -60,6 +60,17 @@ void AddMeSuccess(char *data, int len)
     return;
 }
 
+void GoodbyeCmd(char *data, int len)
+{
+    /* This could probably be greatly improved - I am not sure if anything
+     * needs to be saved here, but certainly it should be possible to
+     * reconnect to the server or a different server without having to
+     * rerun the client.
+     */
+    fprintf(stderr,"Received goodbye command from server - exiting\n");
+    exit(0);
+}
+
 Animations animations[MAXANIM];
 
 void AnimCmd(unsigned char *data, int len)
@@ -102,7 +113,7 @@ void PixMapCmd(unsigned char *data,  int len)
 		(len-8),plen);
 	return;
     }
-    display_newpixmap(pnum,data+8,plen);
+    display_newpixmap(pnum,(char*)data+8,plen);
 }
 
 void BitMapCmd(unsigned char *data, int len)
@@ -115,7 +126,7 @@ void BitMapCmd(unsigned char *data, int len)
 		24*3,len-6);
 	return;
     }
-    display_newbitmap(pnum,data[4],data[5],data+6);
+    display_newbitmap(pnum,data[4],data[5],(char*)data+6);
 }
 
 
@@ -172,14 +183,14 @@ void StatsCmd(unsigned char *data, int len)
 
 	    case CS_STAT_RANGE: {
 		int rlen=data[i++];
-		strncpy(cpl.range,data+i,rlen);
+		strncpy(cpl.range,(const char*)data+i,rlen);
 		cpl.range[rlen]='\0';
 		i += rlen;
 		break;
 	    }
 	    case CS_STAT_TITLE: {
 		int rlen=data[i++];
-		strncpy(cpl.title,data+i,rlen);
+		strncpy(cpl.title,(const char*)data+i,rlen);
 		cpl.title[rlen]='\0';
 		i += rlen;
 		break;
@@ -265,7 +276,7 @@ void PlayerCmd(unsigned char *data, int len)
     face= GetInt_String(data+i);
     i+=4;
     nlen=data[i++];
-    strncpy(name, data+i, nlen);
+    strncpy(name, (const char*)data+i, nlen);
     name[nlen]='\0';
     i+= nlen;
 
@@ -278,6 +289,7 @@ void PlayerCmd(unsigned char *data, int len)
 
 void item_actions (item *op)
 {
+    if (!op) return;
     if (op->open) {
 	open_container (op);
     } else if (op->was_open) {
@@ -445,6 +457,7 @@ void UpdateItemCmd(unsigned char *data, int len)
      * already determined some of the values in this function.
      */
     update_item (tag, loc, name, weight, face, flags,anim,animspeed,nrof);
+    item_actions (locate_item(tag));
 }
 
 void DeleteItem(unsigned char *data, int len)
@@ -463,7 +476,7 @@ void DeleteInventory(unsigned char *data, int len)
 {
     int tag;
 
-    tag=atoi(data);
+    tag=atoi((const char*)data);
     if (tag<0) {
 	fprintf(stderr,"DeleteInventory: Invalid tag: %d\n", tag);
 	return;
@@ -551,7 +564,7 @@ void MagicMapCmd(unsigned char *data, int len)
     int i;
 
     /* First, extract the size/position information. */
-    if (sscanf(data,"%hd %hd %hd %hd", &cpl.mmapx, &cpl.mmapy, 
+    if (sscanf((const char*)data,"%hd %hd %hd %hd", &cpl.mmapx, &cpl.mmapy, 
 	&cpl.pmapx, &cpl.pmapy)!=4) {
 	fprintf(stderr,"Was not able to properly extract magic map size, pos\n");
 	return;
