@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_sound_c =
- *   "$Id: sound.c,v 1.2 1998/04/16 01:39:12 master Exp $";
+ *   "$Id: sound.c,v 1.2 1999/07/12 06:08:03 cvs Exp $";
  */
 
 /*
@@ -19,6 +19,7 @@
 #include <string.h>
 #include <stdlib.h>
 #include <ctype.h>
+#include <signal.h>
 
 #ifndef SOUNDTEST
 #include "client.h"
@@ -28,10 +29,15 @@
 #include "newclient.h"
 #include "soundsdef.h"
 
+
+int nosound=0;
+
+#ifndef NEW_SOUND
 #ifdef linux
 #include <sys/soundcard.h>
 #endif
 
+/*#define SOUND_DEBUG*/
 /* Change if appropriate.  Should probably put this in #ifdefs for various
  * machine types.
  */
@@ -85,11 +91,12 @@ static char volume_table[MAX_VOLUME+1] = {
 85,86,86,87,87,88,88,89,89,90,	/* 90 */
 91,92,93,94,95,96,97,98,99,100 };   /* 100 */
 
+
 /*
  * Nothing after here should be modified - start of data structures we use.
  */
 
-int soundfd=-1,mixerfd=-1, volumecontrol=0, stereo=0, nosound=0;
+int soundfd=-1,mixerfd=-1, volumecontrol=0, stereo=0;
 extern int errno;
 
 typedef struct Sound_Info {
@@ -135,8 +142,10 @@ static void parse_sound_line(char *line, int lineno) {
 	return;
     }
     if (!readtype) {
+#ifdef SOUND_DEBUG
 	fprintf(stderr,"Got input without finding section header yet:\n%d:%s\n",
 		lineno, line);
+#endif
 	return;
     }
 
@@ -188,8 +197,10 @@ static void parse_sound_line(char *line, int lineno) {
 	}
     }
     if (newnum < 0 || newnum>MAX_SOUNDS) {
+#ifdef SOUND_DEBUG
 	fprintf(stderr,"Invalid sound number %d, line %d, buf %s\n",
 		newnum, lineno, line);
+#endif
 	return;
     }
 
@@ -222,6 +233,19 @@ static void parse_sound_line(char *line, int lineno) {
 }
 #endif
 
+#else  /* NEW_SOUND */
+
+/* Got a pipe signal.  As of now, only thing we are piped to is the
+ * sound client.
+ */
+void signal_pipe(int i) {
+    /* do nothing, but perhaps do something more in the future */
+}
+
+FILE *sound_pipe;
+
+#endif /* NEW_SOUND */
+
 /* init_sounds open the audio device, and reads any configuration files
  * that need to be.  It returns 0 on success.  On failure, the calling
  * function will likely disable sound support/requests from the server.
@@ -229,16 +253,19 @@ static void parse_sound_line(char *line, int lineno) {
 
 int init_sounds()
 {
+#ifndef NEW_SOUND
     int i,mask;
     FILE *fp;
     char path[256], buf[512];
+#endif 
 
     /* Easy trick - global nosound is set in the arg processing - if set,
      * just return -1 - this way, the calling function only needs to check
      * the value of init_sounds, and not worry about checking nosound.
      */
     if (nosound) return -1;
-
+    
+#ifndef NEW_SOUND
     soundfd=open(AUDIODEV, O_WRONLY|O_NDELAY);
     if (soundfd==-1) {
 	fprintf(stderr,"Unable to open %s, errno %d\n", AUDIODEV, errno);
@@ -318,9 +345,21 @@ int init_sounds()
 	}
     }
 #endif
+
+#else /* NEW_SOUND */
+
+    sound_pipe=popen("cfsndserv","w");
+    if (!sound_pipe){
+      perror("cfsndserver");
+      return -1;
+    }
+    signal(SIGPIPE, signal_pipe);
+
+#endif /* NEW_SOUND */
     return 0;
 }
 
+
 /* Plays sound 'soundnum'.  soundtype is 0 for normal sounds, 1 for
  * spell_sounds.  This might get extended in the future.  x,y are offset
  * (assumed from player) to play sound.  This information is used to 
@@ -332,6 +371,7 @@ int init_sounds()
 
 static void play_sound(int soundnum, int soundtype, int x, int y)
 {
+#ifndef NEW_SOUND
     int readfd=-1,writecount=0, readcount=0, volume=0,offset;
     char soundbuf[SOUND_BUF];
     Sound_Info *si;
@@ -382,8 +422,10 @@ static void play_sound(int soundnum, int soundtype, int x, int y)
 	    si->size = statbuf.st_size;
 	}
 	if (bufinfo.bytes < si->size) {
+#ifdef SOUND_DEBUG
 	    fprintf(stderr,"sound: buffers %d/%d - won't writed\n", 
 		bufinfo.bytes, si->size);
+#endif
 	    return;
 	}
     }
@@ -438,7 +480,9 @@ static void play_sound(int soundnum, int soundtype, int x, int y)
 		    rightvol=volume;
 		}
 	    }
+#ifdef SOUND_DEBUG
 	    fprintf(stderr,"Stereo volumes = %d/%d\n", leftvol, rightvol);
+#endif
 	    if (leftvol>MAX_VOLUME) leftvol=MAX_VOLUME;
 	    if (rightvol>MAX_VOLUME) rightvol=MAX_VOLUME;
 	    volume = volume_table[leftvol] | (volume_table[rightvol]<<8);
@@ -457,14 +501,18 @@ static void play_sound(int soundnum, int soundtype, int x, int y)
 	fprintf(stderr,"Unable to open %s, error %d\n", si->filename, errno);
 	return;
     }
+#ifdef SOUND_DEBUG
     fprintf(stderr,"Playing sound %s at volume 0x%02x\n", si->filename, volume);
+#endif
 
     /* Skip over the garbage we don't want.  This only applies for
      * for au files, which is all we currently play.
      */
     read(readfd,soundbuf,8);
     offset = soundbuf[4]<<24 | soundbuf[5]<<16 | soundbuf[6]<<8 | soundbuf[7];
+#ifdef SOUND_DEBUG
     fprintf(stderr,"Sound: offset is %x\n", offset);
+#endif
     read(readfd, soundbuf, offset);
 
     /* fragmentsize should be optimum for the sound device driver */
@@ -484,6 +532,19 @@ static void play_sound(int soundnum, int soundtype, int x, int y)
     if (ioctl(soundfd, SNDCTL_DSP_POST, 0)==-1) 
 	fprintf(stderr,"Error with DSP_POST ioctl\n");
 #endif
+#else /* NEW_SOUND */
+    if (nosound) return;
+    if (fprintf(sound_pipe,"%4x %4x %4x %4x\n",soundnum,soundtype,x,y)<=0) {
+	nosound=1;
+	fclose(sound_pipe);
+	return;
+    }
+   if (fflush(sound_pipe)!=0) {
+	nosound=1;
+	fclose(sound_pipe);
+	return;
+   }
+#endif /* NEW_SOUND */
 }
 
 /* SOUNDTEST is defined if you do a 'make sound'.  This creates a standalone
