@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_xio_c =
- *   "$Id: x11.c,v 1.15 1998/05/16 01:44:57 master Exp $";
+ *   "$Id: x11.c,v 1.2 1999/07/12 06:08:03 cvs Exp $";
  *
  * This file handles all the windowing stuff.  The idea is
  * that all of it is in one file, so to port to different systems
@@ -35,7 +35,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    The author can be reached via e-mail to master@rahul.net
+    The author can be reached via e-mail to mark@pyramid.com
 */
 
 /* Most functions in this file are private.  Here is a list of
@@ -73,10 +73,12 @@
 #include <client.h>
 #include <clientbmap.h>
 #include <item.h>
+#include <config.h>
 
-#ifdef Xpm_Pix
+#ifdef HAVE_LIBXPM
 #include <X11/xpm.h>
 #endif
+
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 
@@ -150,8 +152,9 @@ static char font_text_stats[]="8x13", font_text_info[]="8x13",
 #define SCROLLBAR_WIDTH	16	/* +2+2 for border on each side */
 #define INFOCHARS 50
 #define INFOLINES 36
-#define FONTWIDTH 8
-#define FONTHEIGHT 13
+/* Perhaps decent defaults, but not quite right */
+static int  FONTWIDTH= 8;
+static int FONTHEIGHT= 13;
 #define MAX_INFO_WIDTH 80
 #define MAXNAMELENGTH 50
 #define WINUPPER (-5)
@@ -159,6 +162,15 @@ static char font_text_stats[]="8x13", font_text_info[]="8x13",
 #define WINLEFT (-5)
 #define WINRIGHT 5
 
+/* Width (and height) of the game window */
+#define GAME_WIDTH  269
+/* Width of the inventory and look window */
+#define INV_WIDTH   300
+/* spacing between windows */
+#define WINDOW_SPACING	3
+/* Height of the master (root) window */
+#define ROOT_HEIGHT	482
+
 static int gargc;
 
 Display_Mode display_mode = Xpm_Display;
@@ -233,6 +245,11 @@ struct PixmapInfo {
 static struct Map the_map;
 static char stats_buff[7][600];
 static struct PixmapInfo pixmaps[MAXPIXMAPNUM];
+/* Off the 'free' space in the window, this floating number is the
+ * portion that the info takes up.
+ */
+static float info_ratio=0;
+
 #define INFOLINELEN 500
 #define XPMGCS 100
 
@@ -259,66 +276,117 @@ static Stats last_stats = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
 static Window win_root, win_game,win_stats,win_message;
 /* info win */
 #define INFOCHARS 50
-#define FONTWIDTH 8
 
 #include <xutil.c>
 
+/* This is the loop that the client goes through once all the
+ * initialization is done.  Basically, it checks for input and
+ * processes X events (calls function to do that.)
+ * The time for command_loop is fairly arbitrary - it can be most
+ * any value.  If it is very low, however, as it will be doing a lot
+ * of checks to see if there is data instead of blocking on input.
+ *
+ * check_x_events takes all the events that are waiting.
+ */
+
+extern int maxfd;
+void event_loop()
+{
+    fd_set tmp_read, tmp_exceptions;
+    int pollret;
+    struct timeval timeout;
+
+
+    if (MAX_TIME==0) {
+	timeout.tv_sec = 0;
+	timeout.tv_usec = 0;
+    }
+    maxfd = csocket.fd + 1;
+    while (1) {
+	FD_ZERO(&tmp_read);
+	FD_ZERO(&tmp_exceptions);
+	FD_SET(csocket.fd, &tmp_read);
+	FD_SET(csocket.fd, &tmp_exceptions);
+	if (MAX_TIME!=0) {
+	    timeout.tv_sec = MAX_TIME / 1000000;
+	    timeout.tv_usec = MAX_TIME % 1000000;
+	}
+	pollret = select(maxfd, &tmp_read, NULL, NULL, &timeout);
+	if (pollret==-1) {
+	    fprintf(stderr, "Got errno %d on select call.\n", errno);
+	}
+	else if (FD_ISSET(csocket.fd, &tmp_read)) {
+	    DoClient(&csocket);
+	}
+	animate_objects();  /* Do this before the x events, since they
+			     * can redraw this for us.
+			     */
+	check_x_events();
+    }
+}
+
 int misses=0,total=0,newimages=0;
 
 static void gen_draw_face(Drawable where,int face,int x,int y)
 {
-  if (display_mode == Pix_Display) {
-    XSetForeground(display,gc_game,discolor[pixmaps[face].fg].pixel);
-    XSetBackground(display,gc_game,discolor[pixmaps[face].bg].pixel);
-    XCopyPlane(display,pixmaps[face].bitmap,where,gc_game,
+    if (face<0) {
+	fprintf(stderr,"Invalid face number: %d @ %d, %d\n", face, x, y);
+	return;
+    }
+    if (display_mode == Pix_Display) {
+	if (iscolor) {
+	    XSetForeground(display,gc_game,discolor[pixmaps[face].fg].pixel);
+	    XSetBackground(display,gc_game,discolor[pixmaps[face].bg].pixel);
+	}
+	XCopyPlane(display,pixmaps[face].bitmap,where,gc_game,
 	     0,0,24,24,x,y,1);
-  } else if (display_mode == Xpm_Display && pixmaps[face].mask == None) {
-	XCopyArea(display, pixmaps[face].pixmap,
+    } else if (display_mode == Xpm_Display && pixmaps[face].mask == None) {
+	    XCopyArea(display, pixmaps[face].pixmap,
 	      where,gc_floor,
 	      0,0,24,24,x,y);
-  } else if (display_mode == Xpm_Display) {
-    /* Basically, what it looks like all this does is to try and preserve
-     * gc's with various clipmasks set.
-     */
-    int gcnum,i;
-    Pixmap mask;
-    GC gc;
-    total++;
-    mask = pixmaps[face].mask;
-    /* Lets see if we can find a stored mask with matching gc */
-    for(gcnum=0;gcnum<XPMGCS;gcnum++) {
-	if (xpm_masks[gcnum] == mask)
-	break;
-    }
-    /* Nope - we didn't. set one up, but only temporarily */
-    if (gcnum == XPMGCS) {
-	misses++;
-	gcnum--;
+    } else if (display_mode == Xpm_Display) {
+	/* Basically, what it looks like all this does is to try and preserve
+	 * gc's with various clipmasks set.
+	 */
+	int gcnum,i;
+	Pixmap mask;
+	GC gc;
+	total++;
+	mask = pixmaps[face].mask;
+	/* Lets see if we can find a stored mask with matching gc */
+	for(gcnum=0;gcnum<XPMGCS;gcnum++) {
+	    if (xpm_masks[gcnum] == mask)
+	    break;
+	}
+	/* Nope - we didn't. set one up, but only temporarily */
+	if (gcnum == XPMGCS) {
+	    misses++;
+	    gcnum--;
+	    gc = gc_xpm[gcnum];
+	    XSetClipMask(display,gc,mask);
+	}
 	gc = gc_xpm[gcnum];
-	XSetClipMask(display,gc,mask);
-    } 
-    gc = gc_xpm[gcnum];
-    /* Now, we move all the ones up one, and then place the one just used
-     * at position 0.  Thus, the one in the last position was the least
-     * used entry
-     */
-    for(i=gcnum-1;i>=0;i--) {
-	xpm_masks[i+1] = xpm_masks[i];
-	gc_xpm[i+1] = gc_xpm[i];
-    }
-    xpm_masks[0] = mask;
-    gc_xpm[0] = gc;
-    /* Hopefully, this isn't too costly - needed for the inventory and look
-     * window drawing code.
-     */
-    XSetClipOrigin(display, gc_xpm[0], x, y);
-    XCopyArea(display, pixmaps[face].pixmap,
+	/* Now, we move all the ones up one, and then place the one just used
+	 * at position 0.  Thus, the one in the last position was the least
+	 * used entry
+	 */
+	for(i=gcnum-1;i>=0;i--) {
+	    xpm_masks[i+1] = xpm_masks[i];
+	    gc_xpm[i+1] = gc_xpm[i];
+	}
+	xpm_masks[0] = mask;
+	gc_xpm[0] = gc;
+	/* Hopefully, this isn't too costly - needed for the inventory and look
+	 * window drawing code.
+	 */
+	XSetClipOrigin(display, gc_xpm[0], x, y);
+	XCopyArea(display, pixmaps[face].pixmap,
 	    where,gc_xpm[0],
 	    0,0,24,24,x,y);
-  } else {
-    fprintf(stderr,"Unknown display mode %d\n",display_mode);
-    abort();
-  }
+    } else {
+	fprintf(stderr,"Unknown display mode %d\n",display_mode);
+	abort();
+    }
 }
 
 void end_windows()
@@ -341,133 +409,23 @@ void end_windows()
     XCloseDisplay(display);
 }
 
-/* get_root_display: 
- * this sets up the root window (or none, if in split
- * windows mode, and also scans for any Xdefaults.  Right now, only
- * splitwindow and image are used.  image is the display
- * mechanism to use.  I thought having one type that is set
- * to font, xpm, or pixmap was better than using xpm and pixmap
- * resources with on/off values (which gets pretty weird
- * if one of this is set to off.
- */
-
-/* Error handlers removed.  Right now, there is nothing for
- * the client to do if it gets a fatal error - it doesn't have
- * any information to save.  And we might as well let the standard
- * X11 error handler handle non fatal errors.
- */
-int sync_display = 0;
-static int get_root_display(char *display_name) {
-    char *cp;
-    XSizeHints roothint;
-    static char errmsg[MAX_BUF];
-
-    display=XOpenDisplay(display_name);
 
 
-    if (!display) {
-	sprintf(errmsg, "Can't open display %s.", display_name);
-	return 1;
-    }
-    if ((getenv("ERIC_SYNC")!= NULL) || sync_display)
-	XSynchronize(display,True);
 
-    /* For both split_windows and display mode, check to make sure that
-     * the command line has not set the value.  Command line settings
-     * should always have precedence over settings in the Xdefaults file.
-     *
-     * Also, if you add new defaults, make sure that the same rules for
-     * the command line defaults are used.  The client assumes that certain
-     * values can only be set if they have meaning (save_data will not
-     * be set unless the data can actually be saved, for example.)  If
-     * this is not followed for then the client might very well crash.
-     */
-    if (!split_windows && 
-      (cp=XGetDefault(display,X_PROG_NAME,"splitwindow")) != NULL) {
-	if (!strcmp("on",cp) || !strcmp("yes",cp))
-	    split_windows = TRUE;
-	else if (!strcmp("off",cp) || !strcmp("no",cp))
-	    split_windows = FALSE;
-    }
-    if (!cpl.echo_bindings &&
-      (cp=XGetDefault(display,X_PROG_NAME,"echo")) != NULL) {
-	if (!strcmp("on",cp) || !strcmp("yes",cp))
-	    cpl.echo_bindings = TRUE;
-	else if (!strcmp("off",cp) || !strcmp("no",cp))
-	    cpl.echo_bindings = FALSE;
-    }
-    if (display_mode==Font_Display &&
-      (cp=XGetDefault(display,X_PROG_NAME, "image")) != NULL) {
-	if (!strcmp("pixmap",cp))
-	    display_mode = Pix_Display;
-	else if (!strcmp("xpm",cp))
-	    display_mode = Xpm_Display;
-	else if (!strcmp("font",cp))
-	    display_mode = Font_Display;
-	else
-	    fprintf(stderr,"Warning: Unknown image option: %s\n", cp);
-    }
-    if ((cp=XGetDefault(display,X_PROG_NAME, "scrollLines"))!=NULL) {
-	infodata.maxlines=atoi(cp);
-    }
-
-    screen_num=DefaultScreen(display);
-    background=WhitePixel(display,screen_num);
-    foreground=BlackPixel(display,screen_num);
-    roothint.x=0;
-    roothint.y=0;
-    roothint.width=582+6+INFOCHARS*FONTWIDTH;
-    roothint.height=482;
-    roothint.max_width=roothint.min_width=roothint.width;
-    roothint.max_height=roothint.min_height=roothint.height;
-    roothint.flags=PPosition | PSize;
-
-    if(!split_windows) {
-	win_root=XCreateSimpleWindow(display,DefaultRootWindow(display),
-	    roothint.x,roothint.y,roothint.width,roothint.height,2,
-	    background,foreground);
-
-	iscolor=allocate_colors(display, win_root, screen_num,
-	    &colormap, discolor);
-	if (iscolor){
-	    foreground=discolor[0].pixel;
-	    background=discolor[8].pixel;
-	}
-	icon=XCreateBitmapFromData(display,win_root,
-	    (_Xconst char *) crossfire_bits,
-	    (unsigned int) crossfire_width, (unsigned int)crossfire_height);
-	XSetStandardProperties(display,win_root,X_PROG_NAME,X_PROG_NAME,
-	    icon,gargv,gargc,&(roothint));
-	gc_root=XCreateGC(display,win_root,0,0);
-	XSetForeground(display,gc_root,foreground);
-	XSetBackground(display,gc_root,background);
-    }
-    else win_root = DefaultRootWindow(display);
-
-    if(display_mode==Font_Display)
-	(void) fixfontpath(display,"client");
-
-    if(!split_windows) {
-	if(display_mode==Font_Display) {
-	    font=XLoadFont(display,font_graphic);
-	    XSetFont(display,gc_root,font);
-	}
-	XSelectInput(display,win_root,KeyPressMask|
-	     KeyReleaseMask|ExposureMask|StructureNotifyMask);
-	XMapRaised(display,win_root);
-	XNextEvent(display,&event);
-    }
-    return 0;
-}
 
+/***********************************************************************
+ *
+ * Stats window functions follow
+ *
+ ***********************************************************************/
 
 static int get_game_display() {
     XSizeHints gamehint;
    
-    gamehint.x=305;
+    gamehint.x=INV_WIDTH + WINDOW_SPACING;
     gamehint.y=104;
-    gamehint.width=269;
-    gamehint.height=269;
+    gamehint.width=GAME_WIDTH;
+    gamehint.height=GAME_WIDTH;
     gamehint.max_width=gamehint.min_width=gamehint.width;
     gamehint.max_height=gamehint.min_height=gamehint.height;
     gamehint.flags=PPosition | PSize;
@@ -543,10 +501,11 @@ static int get_game_display() {
 static int get_info_display() {
     XSizeHints infohint;
     int i;
+    XFontStruct	*xfs;
 
     /* The following could happen if bad values are given. */
     if (infodata.maxlines<INFOLINES) infodata.maxlines=INFOLINES;
-    infohint.x=579;
+    infohint.x=INV_WIDTH + GAME_WIDTH + WINDOW_SPACING*2;
     infohint.y=0;
     infohint.width=infodata.width=6+INFOCHARS*FONTWIDTH;
     infohint.height=infodata.height=11+infodata.maxdisp*13;
@@ -565,8 +524,23 @@ static int get_info_display() {
     infodata.gc_info=XCreateGC(display,infodata.win_info,0,0);
     XSetForeground(display,infodata.gc_info,foreground);
     XSetBackground(display,infodata.gc_info,background);
-    font=XLoadFont(display,font_text_info);
-    XSetFont(display,infodata.gc_info,font);
+    xfs=XLoadQueryFont(display,font_text_info);
+
+    XSetFont(display,infodata.gc_info,xfs->fid);
+    i=xfs->max_bounds.rbearing - xfs->min_bounds.lbearing;
+    if (i != FONTWIDTH) {
+	fprintf(stderr,"Font has actual width of %d, resetting from %d\n",
+		i, FONTWIDTH);
+	FONTWIDTH=i;
+    }
+
+    i=xfs->max_bounds.ascent + xfs->max_bounds.descent;
+    if (i != FONTHEIGHT) {
+	fprintf(stderr,"Font has actual width of %d, resetting from %d\n",
+		i, FONTHEIGHT);
+	FONTHEIGHT=i;
+    }
+
     XSelectInput(display,infodata.win_info,
 	ButtonPressMask|KeyPressMask|KeyReleaseMask|ExposureMask|
 	StructureNotifyMask);
@@ -784,7 +758,7 @@ void draw_info(const char *str, int color) {
     return;
   }
 
-  /* Lets do the word wrap for messages - MSW (master@rahul.net) */
+  /* Lets do the word wrap for messages - MSW */
   if ((int)strlen(str) >= infodata.info_chars) {
 	int i=infodata.info_chars-1;
 
@@ -985,19 +959,19 @@ static void draw_all_info() {
 }
 
 
-static void resize_win_info(XEvent *event) {
-    int chars=(event->xconfigure.width/FONTWIDTH)-1;
-    int lines=(event->xconfigure.height/FONTHEIGHT);
+static void resize_win_info(int width, int height) {
+    int chars=(width/FONTWIDTH)-1;
+    int lines=(height/FONTHEIGHT)-1;
     int i;
 
-    if (infodata.width==event->xconfigure.width &&
-	infodata.height==event->xconfigure.height/FONTHEIGHT) return;
+    if (infodata.width==width &&
+	infodata.height==height/FONTHEIGHT) return;
 
     if(chars<3 || lines<3)
 	return;
 
-    infodata.width=event->xconfigure.width;
-    infodata.height=event->xconfigure.height;
+    infodata.width=width;
+    infodata.height=height;
     infodata.bar_length=infodata.height - 8;
 
     /* IF the window has shrunk, and if the number of lines equals
@@ -1031,7 +1005,7 @@ static void resize_win_info(XEvent *event) {
 	 */
 	else {
 	    InfoLine *newlines;
-	    int start=infodata.numlines-lines,i,j;
+	    int start=infodata.numlines-lines,i,j=0;
 
 	    if (start<0) start+=infodata.maxlines;
 	    newlines=malloc(sizeof(InfoLine) * (lines+1));
@@ -1131,9 +1105,9 @@ static void resize_win_info(XEvent *event) {
 static int get_stats_display() {
     XSizeHints stathint;
 
-    stathint.x=303;
+    stathint.x=INV_WIDTH + WINDOW_SPACING;
     stathint.y=0;
-    stathint.width=273;
+    stathint.width=GAME_WIDTH;
     stathint.height=100;
     stathint.min_width=stathint.max_width=stathint.width;
     stathint.min_height=stathint.max_height=stathint.height;
@@ -1166,10 +1140,7 @@ static int get_stats_display() {
 void draw_stats(int redraw) {
   char buff[MAX_BUF];
   static char last_name[MAX_BUF]="", last_range[MAX_BUF]="";
-#if 0
-  static uint8 last_spell;
-  static rangetype last_shoottype;
-#endif
+
     if (strcmp(cpl.title, last_name) || redraw) {
 	strcpy(last_name,cpl.title);
 	strcpy(buff,cpl.title);
@@ -1196,7 +1167,7 @@ void draw_stats(int redraw) {
        cpl.stats.grace!=last_stats.grace || 
        cpl.stats.maxgrace!=last_stats.maxgrace) {
 
-	sprintf(buff,"Hp: %d/%d  Sp: %d/%d Gr: %d/%d",
+	sprintf(buff,"Hp %d/%d  Sp %d/%d Gr %d/%d",
 	    cpl.stats.hp, cpl.stats.maxhp,
 	    cpl.stats.sp, cpl.stats.maxsp,
 	    cpl.stats.grace, cpl.stats.maxgrace);
@@ -1261,17 +1232,27 @@ void draw_stats(int redraw) {
 	 * by speed - the multiplication/division factor on both factors
 	 * out.
 	 */
-	float weap_sp = (float) cpl.stats.speed/
-		((float)cpl.stats.weapon_sp);
+	double weap_sp;
 
-	if(cpl.stats.food<100 && (cpl.stats.food&4))
+	/* Seems that weapon_sp can be 0 in some cases which caused SIGFPE's */
+	if (cpl.stats.weapon_sp ==0) weap_sp = 0;
+	else weap_sp = (float) cpl.stats.speed/	((float)cpl.stats.weapon_sp);
+
+	/* The following is generating an FPE on alpha systems - changed
+	 * everything to be doubles to see if that might make some
+	 * difference.
+	 */
+	if(cpl.stats.food<100 && (cpl.stats.food&4)) {
 	    sprintf(buff,"Speed: %3.2f (%1.2f) Food: *%d* HUNGRY!",
-		(float)cpl.stats.speed/FLOAT_MULTF,
+		(double)cpl.stats.speed/FLOAT_MULTF,
 		weap_sp,cpl.stats.food);
-	else
+	    if (cpl.food_beep && (cpl.stats.food%4==3)) XBell(display, 0);
+	} else {
 	    sprintf(buff,"Speed: %3.2f (%1.2f)  Food: %3d",
 		(float)cpl.stats.speed/FLOAT_MULTF,
 		weap_sp, cpl.stats.food);
+	    if (cpl.food_beep && cpl.stats.food<1) XBell(display,100);
+	}
 
  	strcat(buff,"                     ");
 	XDrawImageString(display,win_stats,
@@ -1288,58 +1269,6 @@ void draw_stats(int redraw) {
 	XDrawImageString(display,win_stats,
 	    gc_stats,10,94, buff,strlen(buff));
     }
-#if 0
-    if (redraw || last_shoottype!=cpl.shoottype ||
-      (cpl.shoottype==range_magic && cpl.ready_spell!=last_spell)) {
-
-	switch(cpl.shoottype) {
-	    case range_none:
-		strcpy(buff,"Range: nothing");
-		break;
-
-	    case range_bow: {
-		char buf2[MAX_BUF];
-
-		if (cpl.ranges[range_bow]==NULL) break;
-		strcpy(buf2, cpl.ranges[range_bow]->name);
-		if (strcmp(buf2 + strlen(buf2)-10, " (readied)") == 0)
-		    buf2[strlen(buf2) - 10] = 0;
-	        sprintf (buff, "Range: %s", buf2);
-		break;
-	    }
-	case range_magic:
-	    sprintf(buff,"Range: spell (%s)", cpl.spells[cpl.ready_spell]);
-	    break;
-
-	case range_wand:
-	case range_rod:
-	case range_horn:
-	    sprintf(buff,"Range: %s",
-		cpl.ranges[cpl.shoottype]?
-		cpl.ranges[cpl.shoottype]->name:"null");
-	    break;
-
-	case range_scroll:
-	    sprintf(buff,"Range: golem (%s)",
-		cpl.ranges[cpl.shoottype]?
-		cpl.ranges[cpl.shoottype]->name:"null");
-	    break;
-
-	    case range_steal:
-		strcpy(buff, "Range: steal");
-		break;
-
-	    default:
-		strcpy(buff,"Range: illegal");
-	}
-	last_spell= cpl.ready_spell;
-	last_shoottype=cpl.shoottype;
-
- 	strcat(buff,"                     ");
-	XDrawImageString(display,win_stats,
-	    gc_stats,10,94, buff,strlen(buff));
-      }
-#endif
 }
 
 /***********************************************************************
@@ -1351,9 +1280,9 @@ void draw_stats(int redraw) {
 static int get_message_display() {
     XSizeHints messagehint;
 
-    messagehint.x=303;
+    messagehint.x=INV_WIDTH + WINDOW_SPACING;
     messagehint.y=378;
-    messagehint.width=273;
+    messagehint.width=GAME_WIDTH;
     messagehint.height=101;
     messagehint.max_width=messagehint.min_width=messagehint.width;
     messagehint.max_height=messagehint.min_height=messagehint.height;
@@ -1582,7 +1511,7 @@ static void create_status_icons ()
 #include "pixmaps/close.xpm"
 #include "pixmaps/stipple.111"
 #include "pixmaps/stipple.112"
-    static hasinit=0;
+    static int hasinit=0;
 
     if (hasinit) return;
     hasinit=1;
@@ -1925,7 +1854,9 @@ static int get_inv_display()
     inv_list.show_weight = 1;
     inv_list.show_what = show_all;
     inv_list.weight_limit=0;
-    get_list_display ( &inv_list, 0, 0, 300, 310, "Crossfire - inventory",
+    get_list_display ( &inv_list, 0, 0, INV_WIDTH, 
+		      2*(ROOT_HEIGHT - WINDOW_SPACING) / 3, 
+		      "Crossfire - inventory",
 		      "crossinventory");
     return 0;
 }
@@ -1937,7 +1868,11 @@ static int get_look_display()
     look_list.show_weight = 1;
     look_list.show_what = show_all;
     inv_list.weight_limit = 0;
-    get_list_display ( &look_list, 0, 313, 300, 166, "Crossfire - look",
+    get_list_display ( &look_list, 0, 
+	      (2*(ROOT_HEIGHT - WINDOW_SPACING) / 3) + WINDOW_SPACING,
+		      INV_WIDTH, 
+		      (ROOT_HEIGHT - WINDOW_SPACING) / 3,
+		      "Crossfire - look",
 		    "crosslook");
     return 0;
 }
@@ -2008,7 +1943,7 @@ void set_scroll(char *s)
 void clear_infodata.win_info(object *op) {
   int i;
 
-	/* don't clear if in scroll mode MSW (master@rahul.net) */
+	/* don't clear if in scroll mode MSW */
   if((op->type==PLAYER)  &&  (op->contr->scroll)) return;
 
   for(i=0;i<op->contr->maxinfodata.infolines;i++) {
@@ -2028,13 +1963,177 @@ int get_info_width()
     return infodata.info_chars;
 }
 
-/* Magic mapping code needs to be added.
- * The way I see it, server will send along data of the squares and what
- * color they should be in single character codes.  So if we are mapping a
- * 10x10 area, it would be something like: "1155123620", 1296639403",
- * etc.
+/******************************************************************************
+ * Root Window code
+ ****************************************************************************/
+
+/* get_root_display: 
+ * this sets up the root window (or none, if in split
+ * windows mode, and also scans for any Xdefaults.  Right now, only
+ * splitwindow and image are used.  image is the display
+ * mechanism to use.  I thought having one type that is set
+ * to font, xpm, or pixmap was better than using xpm and pixmap
+ * resources with on/off values (which gets pretty weird
+ * if one of this is set to off.
  */
 
+/* Error handlers removed.  Right now, there is nothing for
+ * the client to do if it gets a fatal error - it doesn't have
+ * any information to save.  And we might as well let the standard
+ * X11 error handler handle non fatal errors.
+ */
+int sync_display = 0;
+
+static int get_root_display(char *display_name) {
+    char *cp;
+    XSizeHints roothint;
+    static char errmsg[MAX_BUF];
+
+    display=XOpenDisplay(display_name);
+
+
+    if (!display) {
+	sprintf(errmsg, "Can't open display %s.", display_name);
+	return 1;
+    }
+    if ((getenv("ERIC_SYNC")!= NULL) || sync_display)
+	XSynchronize(display,True);
+
+    /* For both split_windows and display mode, check to make sure that
+     * the command line has not set the value.  Command line settings
+     * should always have precedence over settings in the Xdefaults file.
+     *
+     * Also, if you add new defaults, make sure that the same rules for
+     * the command line defaults are used.  The client assumes that certain
+     * values can only be set if they have meaning (save_data will not
+     * be set unless the data can actually be saved, for example.)  If
+     * this is not followed for then the client might very well crash.
+     */
+    if (!split_windows && 
+      (cp=XGetDefault(display,X_PROG_NAME,"splitwindow")) != NULL) {
+	if (!strcmp("on",cp) || !strcmp("yes",cp))
+	    split_windows = TRUE;
+	else if (!strcmp("off",cp) || !strcmp("no",cp))
+	    split_windows = FALSE;
+    }
+    if (!cpl.echo_bindings &&
+      (cp=XGetDefault(display,X_PROG_NAME,"echo")) != NULL) {
+	if (!strcmp("on",cp) || !strcmp("yes",cp))
+	    cpl.echo_bindings = TRUE;
+	else if (!strcmp("off",cp) || !strcmp("no",cp))
+	    cpl.echo_bindings = FALSE;
+    }
+    if (display_mode==Font_Display &&
+      (cp=XGetDefault(display,X_PROG_NAME, "image")) != NULL) {
+	if (!strcmp("pixmap",cp))
+	    display_mode = Pix_Display;
+	else if (!strcmp("xpm",cp))
+	    display_mode = Xpm_Display;
+	else if (!strcmp("font",cp))
+	    display_mode = Font_Display;
+	else
+	    fprintf(stderr,"Warning: Unknown image option: %s\n", cp);
+    }
+    if ((cp=XGetDefault(display,X_PROG_NAME, "scrollLines"))!=NULL) {
+	infodata.maxlines=atoi(cp);
+    }
+
+    screen_num=DefaultScreen(display);
+    background=WhitePixel(display,screen_num);
+    foreground=BlackPixel(display,screen_num);
+    roothint.x=0;
+    roothint.y=0;
+    roothint.width=582+6+INFOCHARS*FONTWIDTH;
+    roothint.height=ROOT_HEIGHT;
+    roothint.max_width=roothint.min_width=roothint.width;
+    roothint.max_height=roothint.min_height=roothint.height;
+    roothint.flags=PPosition | PSize;
+
+    if(!split_windows) {
+	win_root=XCreateSimpleWindow(display,DefaultRootWindow(display),
+	    roothint.x,roothint.y,roothint.width,roothint.height,2,
+	    background,foreground);
+
+	iscolor=allocate_colors(display, win_root, screen_num,
+	    &colormap, discolor);
+	if (iscolor){
+	    foreground=discolor[0].pixel;
+	    background=discolor[8].pixel;
+	}
+	icon=XCreateBitmapFromData(display,win_root,
+	    (_Xconst char *) crossfire_bits,
+	    (unsigned int) crossfire_width, (unsigned int)crossfire_height);
+	XSetStandardProperties(display,win_root,X_PROG_NAME,X_PROG_NAME,
+	    icon,gargv,gargc,&(roothint));
+	gc_root=XCreateGC(display,win_root,0,0);
+	XSetForeground(display,gc_root,foreground);
+	XSetBackground(display,gc_root,background);
+    }
+    else win_root = DefaultRootWindow(display);
+
+    if(display_mode==Font_Display)
+	(void) fixfontpath(display,"client");
+
+    if(!split_windows) {
+	if(display_mode==Font_Display) {
+	    font=XLoadFont(display,font_graphic);
+	    XSetFont(display,gc_root,font);
+	}
+	XSelectInput(display,win_root,KeyPressMask|
+	     KeyReleaseMask|ExposureMask|StructureNotifyMask);
+	XMapRaised(display,win_root);
+	XNextEvent(display,&event);
+    }
+    return 0;
+}
+
+static void resize_win_root(XEvent *event) {
+    int width, inv_width, info_width;
+
+    if (split_windows) {
+	fprintf(stderr,"Got a resize root window in split windows mode\n");
+	return;
+    }
+
+    /* The middle 3 windows don't really benefit from the resize, so keep
+     * them the same size, and use the leftover equally between the left
+     * and right windows.
+     */
+    width = (event->xconfigure.width - GAME_WIDTH -WINDOW_SPACING*2);
+    info_width = width * info_ratio;
+    inv_width = width - info_width;
+
+    /* These windows just need to be relocated.  The y constants are
+     * hardcoded - those windows don't really benefit from being resized
+     * (actually, no code in place to currently do it), so no reason
+     * to get trick with those just now.
+     */
+    XMoveWindow(display, win_message, inv_width + WINDOW_SPACING, 378);
+    XMoveWindow(display, win_game, inv_width + WINDOW_SPACING, 104);
+    XMoveWindow(display, win_stats, inv_width + WINDOW_SPACING, 0);
+
+    /* Resize the info window */
+    XMoveResizeWindow(display, infodata.win_info,  
+	inv_width + GAME_WIDTH + WINDOW_SPACING * 2, 0,
+	info_width, event->xconfigure.height);
+
+
+    /* Resize the inventory, message window */
+    XResizeWindow(display, inv_list.win, inv_width, 
+		  2 * (event->xconfigure.height - WINDOW_SPACING) / 3);
+
+
+    XMoveResizeWindow(display, look_list.win, 0, 
+	      (2*(event->xconfigure.height - WINDOW_SPACING) / 3) + WINDOW_SPACING,
+		      inv_width, 
+		      (event->xconfigure.height - WINDOW_SPACING) / 3);
+
+    /* The Resize requests will generate events.  As such, we don't call
+     * the resize functions here - the event handler would get those anyways,
+     * so there isn't a big gain in doing it here.
+     */
+}
+
 
 /***********************************************************************
  *
@@ -2078,7 +2177,7 @@ static void parse_game_button_press(int button, int x, int y)
 		      case 8: fire_dir (4);break;
 		}
 		/* Only want to fire once */
-		stop_fire();
+		clear_fire();
 	    }
 	    else switch (i) {
 	      case 0: move_player (8);break;
@@ -2138,9 +2237,13 @@ static void do_key_press()
 	    break;
 
 	case Reply_One:
-	    text[1]='\0';
-	    send_reply(text);
-	    cpl.input_state = Playing;
+	/* Don't send modifier keys as reply to query. Tries to prevent
+	 * from getting into the "...state is not ST_PLAYING" state. */
+	    if (text[0]) {
+		text[1]='\0';
+		send_reply(text);
+		cpl.input_state = Playing;
+	    }
 	    break;
 
 	case Configure_Keys:
@@ -2180,7 +2283,7 @@ static void do_key_press()
 
 static void buttonpress_in_info(XButtonEvent *xbutton)
 {
-    int y = xbutton->y-16, x=xbutton->x, button = xbutton->button,dy,pos;
+    int y = xbutton->y-16, x=xbutton->x, button = xbutton->button,dy,pos=0;
 
     if (!infodata.has_scrollbar || x<=(infodata.width-SCROLLBAR_WIDTH-4))
 	return;
@@ -2226,7 +2329,7 @@ static int buttonpress_in_list (itemlist *l, XButtonEvent *xbutton)
 {
     item *tmp;
     int y = xbutton->y - 16, x=xbutton->x, button = xbutton->button;
-    int items, pos, dy;
+    int items, pos=0, dy;
 
     if (y < 0 && l->env->open) /* close the sack */
 	client_send_apply (l->env->tag);
@@ -2307,7 +2410,7 @@ static int buttonpress_in_list (itemlist *l, XButtonEvent *xbutton)
  */
 
 void check_x_events() {
-  KeySym gkey;
+  KeySym gkey=0;
   static int lastupdate=0;
 
   /* We need to periodically redraw stuff if we are caching images - otherwise
@@ -2337,13 +2440,15 @@ void check_x_events() {
 
     case ConfigureNotify:
       if(event.xconfigure.window==infodata.win_info)
-	resize_win_info(&event);
+	resize_win_info(event.xconfigure.width, event.xconfigure.height);
       else if(event.xconfigure.window==inv_list.win)
 	resize_list_info(&inv_list, event.xconfigure.width,
 			 event.xconfigure.height);
       else if(event.xconfigure.window==look_list.win)
 	resize_list_info(&look_list, event.xconfigure.width,
 			 event.xconfigure.height);
+      else if(event.xconfigure.window==win_root)
+	resize_win_root(&event);
       break;
 
     case Expose:
@@ -2410,6 +2515,7 @@ void check_x_events() {
 
     /* Need to do this to show the players position */
     if (cpl.showmagic) magic_map_flash_pos();
+    clear_fire_run();
 }
 
 
@@ -2433,16 +2539,18 @@ static void usage(char *progname)
     puts("-port <number>   - Use port <number> instead of the standard port number");
     puts("-display <name>  - Use <name> instead if DISPLAY environment variable.\n");
     puts("-split           - Use split windows.");
+    puts("-nosplit         - Disable split windows (default action).");
     puts("-echo            - Echo the bound commands");
     puts("-pix             - Use bitmaps instead of the font.");
-#ifdef Xpm_Pix
+#ifdef HAVE_LIBXPM
     puts("-xpm             - Use color pixmaps (XPM) for display.");
 #endif
     puts("-showicon        - Print status icons in inventory window");
     puts("-scrolllines <number>    - number of lines for scrollback");
-    puts("-image           - get all images from server at startup");
     puts("-sync            - Synchronize on display");
     puts("-help            - Display this message.");
+    puts("-cache           - Cache images for future use.");
+    puts("-nocache         - Do not cache images (default action).");
     exit(0);
 }
 
@@ -2495,7 +2603,7 @@ int init_windows(int argc, char **argv)
 	    continue;
 	}
 	if (!strcmp(argv[on_arg],"-xpm")) {
-#ifdef Xpm_Pix
+#ifdef HAVE_LIBXPM
 	    display_mode = Xpm_Display;
 	    continue;
 #else
@@ -2512,10 +2620,18 @@ int init_windows(int argc, char **argv)
 	    cache_images= TRUE;
 	    continue;
 	}
+	else if (!strcmp(argv[on_arg],"-nocache")) {
+	    cache_images= FALSE;
+	    continue;
+	}
 	else if (!strcmp(argv[on_arg],"-split")) {
 	    split_windows=TRUE;
 	    continue;
 	}
+	else if (!strcmp(argv[on_arg],"-nosplit")) {
+	    split_windows=FALSE;
+	    continue;
+	}
 	else if (!strcmp(argv[on_arg],"-showicon")) {
 	    inv_list.show_icon = TRUE;
 	    continue;
@@ -2564,6 +2680,7 @@ int init_windows(int argc, char **argv)
     init_keys();
     if (cache_images) init_cache_data();
     set_window_pos();
+    info_ratio=(float) infodata.width/ (float) (infodata.width + INV_WIDTH);
     return 0;
 }
 
@@ -2596,13 +2713,6 @@ void display_mapcell_bitmap(int ax,int ay)
 {
   gen_draw_face(win_game,the_map.cells[ax][ay].faces[0],
 		2+24*ax,2+24*ay);
-#if 0
-  int face = the_map.cells[ax][ay].faces[0];
-  XSetForeground(display,gc_game,discolor[pixmaps[face].fg].pixel);
-  XSetBackground(display,gc_game,discolor[pixmaps[face].bg].pixel);
-  XCopyPlane(display,pixmaps[face].bitmap,win_game,gc_game,
-	     0,0,24,24,2+24*ax,2+24*ay,1);
-#endif
 }
 
 int display_usebitmaps()
@@ -2674,7 +2784,6 @@ void display_mapscroll(int dx,int dy)
 void display_newpixmap(long face,char *buf,long buflen)
 {
     FILE *tmpfile;
-    char tmpfilename[200];
     Pixmap pixmap, mask;
     XpmAttributes xpm_attr;
     int error;
@@ -2684,37 +2793,24 @@ void display_newpixmap(long face,char *buf,long buflen)
     if (cache_images) {
 	if (facetoname[face]==NULL) {
 	    fprintf(stderr,"Caching images, but name for %ld not set\n", face);
-	    sprintf(tmpfilename,"/tmp/xclient.%d",(int)getpid());
-	    tmpfile = fopen(tmpfilename,"w");
 	}
 	else if ((tmpfile = fopen(facetoname[face],"w"))==NULL) {
 	    fprintf(stderr,"Can not open %s for writing\n", facetoname[face]);
-	    sprintf(tmpfilename,"/tmp/xclient.%d",(int)getpid());
-	    tmpfile = fopen(tmpfilename,"w");
 	}
-	else strcpy(tmpfilename,facetoname[face]);
-    } else { /* not caching images */
-	sprintf(tmpfilename,"/tmp/xclient.%d",(int)getpid());
-	tmpfile = fopen(tmpfilename,"w");
+	else {
+	    fprintf(tmpfile,"%s",buf);
+	    fclose(tmpfile);
+	}
     }
-    fprintf(tmpfile,"%s",buf);
-    fclose(tmpfile);
+    error = XpmCreatePixmapFromBuffer(display, win_game, buf,
+		&pixmap, &mask, &xpm_attr);
 
-    error=XpmReadFileToPixmap(display,win_game,tmpfilename, 
-		&pixmap,&mask,&xpm_attr);
-    newimages++;
-
-    if (error!= XpmSuccess) {
-	fprintf(stderr,"Unable to read %s as pixmap file, error=%d\n",tmpfilename, error);
-	abort();
-    }
 /*  printf("newpixmap #%ld: (%ld,%ld)\n",face,pixmap,mask);*/
     pixmaps[face].pixmap = pixmap;
     pixmaps[face].mask = mask;
+    newimages++;
 
-    if (!cache_images) {
-	unlink(tmpfilename);
-    } else if (facetoname[face]) {
+    if (facetoname[face] && cache_images) {
 	free(facetoname[face]);
 	facetoname[face]=NULL;
     }
