@@ -1,11 +1,17 @@
-#include <X11/keysym.h>
-#include "def-keys.h"
-/* This contains varous 'support' functions.  These functions will probably
+/*
+ * static char *rcsid_xio_c =
+ *   "$Id: xutil.c,v 1.4 1999/07/12 06:08:03 cvs Exp $";
+ *
+ * This contains varous 'support' functions.  These functions will probably
  * go mostly unaltered between different toolkits, as long as X11 is still
  * used.  This file is not compiled seperately, rather it is included by
  * x11.c, so all statics will still work fine.
  */
 
+#include <X11/keysym.h>
+#include "def-keys.h"
+
+
 static char *colorname[] = {
 "Black",                /* 0  */
 "White",                /* 1  */
@@ -56,7 +62,7 @@ static void init_cache_data()
 
     pixmaps[0].mask=None;
     pixmaps[0].bitmap=XCreateBitmapFromData(display, 
-	RootWindow(display, screen_num), question_bits, 24,24);
+	RootWindow(display, screen_num), (const char*)question_bits, 24,24);
 
     /* In xpm mode, XCopyArea is used from this data, so we need to copy
      * the image into an pixmap of appropriate depth.
@@ -120,7 +126,7 @@ void FaceCmd(unsigned char *data,  int len)
 	return;
     }
     pnum = GetShort_String(data);
-    face = data+2;
+    face = (char*)data+2;
     data[len] = '\0';
     /* To prevent having a directory with 2000 images, we do a simple
      * split on the first 2 characters.
@@ -188,6 +194,7 @@ static int allocate_colors(Display *disp, Window w, long screen_num,
     depth = DefaultDepth(disp,screen_num);
   }
   else {
+    *colormap = DefaultColormap(disp,screen_num);
     printf("Switching to black and white.\n");
     printf("You have a black and white terminal.\n");
     return 0;
@@ -523,7 +530,7 @@ static void init_keys()
 {
     int i, line=0;
     FILE *fp;
-    char buf[MAX_BUF];
+    char buf[BIG_BUF];
 
     commandkeysym = XK_apostrophe;
     commandkey =XKeysymToKeycode(display,XK_apostrophe);
@@ -573,7 +580,7 @@ static void init_keys()
 	    return;
 	}
     }
-    while (fgets(buf, MAX_BUF, fp)) {
+    while (fgets(buf, BIG_BUF, fp)) {
 	line++;
 	parse_keybind_line(buf,line,0);
     }
@@ -709,7 +716,7 @@ static void parse_key(char key, KeyCode keycode, KeySym keysym)
     sprintf(buf, "Key unused (%s%s%s)",
           (cpl.fire_on? "Fire&": ""),
           (cpl.run_on ? "Run&" : ""),
-          XKeysymToString(keysym));
+          keysym==NoSymbol? "unknown": XKeysymToString(keysym));
     draw_info(buf,NDI_BLACK);
     cpl.count=0;
 }
@@ -777,16 +784,17 @@ static void show_keys(int allbindings)
   Key_Entry *key;
   char buf[MAX_BUF];
 
-  sprintf(buf, "Commandkey %s (%d)", XKeysymToString(commandkeysym),
-	commandkey);
+  sprintf(buf, "Commandkey %s (%d)", 
+	  commandkeysym==NoSymbol?"unknown":XKeysymToString(commandkeysym),
+	  commandkey);
   draw_info(buf,NDI_BLACK);
   sprintf(buf, "Firekeys 1: %s (%d), 2: %s (%d)",
-	  XKeysymToString(firekeysym[0]), firekey[0],
-	  XKeysymToString(firekeysym[1]), firekey[1]);
+	  firekeysym[0]==NoSymbol?"unknown":XKeysymToString(firekeysym[0]), firekey[0],
+	  firekeysym[1]==NoSymbol?"unknown":XKeysymToString(firekeysym[1]), firekey[1]);
   draw_info(buf,NDI_BLACK);
   sprintf(buf, "Runkeys 1: %s (%d), 2: %s (%d)",
-	  XKeysymToString(runkeysym[0]), runkey[0],
-	  XKeysymToString(runkeysym[1]), runkey[1]);
+	  runkeysym[0]==NoSymbol?"unknown":XKeysymToString(runkeysym[0]), runkey[0],
+	  runkeysym[1]==NoSymbol?"unknown":XKeysymToString(runkeysym[1]), runkey[1]);
   draw_info(buf,NDI_BLACK);
 
 
@@ -816,6 +824,9 @@ void bind_key(char *params)
     return;
   }
 
+  /* Skip over any spaces we may have */
+  while (*params==' ') params++;
+
   if (!strcmp(params, "commandkey")) {
     bind_keycode = &commandkey;
     bind_keysym = &commandkeysym;
@@ -921,6 +932,10 @@ static void save_keys()
     FILE *fp;
 
     sprintf(buf,"%s/.crossfire/keys", getenv("HOME"));
+    if (make_path_to_file(buf)==-1) {
+	fprintf(stderr,"Could not create %s\n", buf);
+	return;
+    }
     if ((fp=fopen(buf,"w"))==NULL) {
 	sprintf(buf2,"Could not open %s, key bindings not saved\n", buf);
 	draw_info(buf2,NDI_BLACK);
@@ -971,7 +986,8 @@ static void configure_keys(KeyCode k, KeySym keysym)
 	insert_key(keysym, k, bind_flags, bind_buf);
   }
 
-  sprintf(buf, "Binded to key '%s' (%i)", XKeysymToString(keysym), (int)k);
+  sprintf(buf, "Binded to key '%s' (%i)", 
+	  keysym==NoSymbol?"unknown":XKeysymToString(keysym), (int)k);
   draw_info(buf,NDI_BLACK);
   cpl.fire_on=0;
   cpl.run_on=0;
@@ -996,6 +1012,10 @@ void unbind_key(char *params)
 	show_keys(0);
 	return;
     }
+
+    /* Skip over any spaces we may have */
+    while (*params==' ') params++;
+
     if (!strcmp(params,"-a")) {
 	show_keys(1);
 	return;
@@ -1081,7 +1101,8 @@ void save_winpos()
 {
     char savename[MAX_BUF],buf[MAX_BUF];
     FILE    *fp;
-    int	    x,y,w,h,wx,wy;
+    int	    x,y,wx,wy;
+    unsigned int w,h;
 
     if (!split_windows) {
 	draw_info("You can only save window positions in split windows mode", NDI_BLUE);
@@ -1215,6 +1236,18 @@ void load_defaults()
 	    else nosound=TRUE;
 	    continue;
 	}
+	if (!strcmp(inbuf,"command_window")) {
+	    cpl.command_window = atoi(cp);
+	    if (cpl.command_window<1 || cpl.command_window>127)
+		cpl.command_window=COMMAND_WINDOW;
+	    continue;
+	}
+	if (!strcmp(inbuf,"foodbeep")) {
+	    if (!strcmp(cp,"True")) cpl.food_beep=TRUE;
+	    else cpl.food_beep=FALSE;
+	    continue;
+	}
+
 	fprintf(stderr,"Got line we did not understand: %s: %s", inbuf, cp);
     }
     fclose(fp);
@@ -1252,6 +1285,8 @@ void save_defaults()
     fprintf(fp,"scrolllines: %d\n", infodata.maxlines);
     fprintf(fp,"scrollinfo: %s\n", infodata.scroll_info_window?"True":"False");
     fprintf(fp,"sound: %s\n", nosound?"False":"True");
+    fprintf(fp,"command_window: %d\n", cpl.command_window);
+    fprintf(fp,"foodbeep: %s\n", cpl.food_beep?"True":"False");
     fclose(fp);
     sprintf(buf,"Defaults saved to %s",path);
     draw_info(buf,NDI_BLUE);
